#include <stdio.h>
#include <stdlib.h>
#include "range.h"

#define BSIZE 1000000
#define SYMS 256

Uint ftbl[SYMS];  /* The frequency table */

/* A simple demonstration */
int main () {
	Sint sym;
	Uint i, n;
	uc *op, *s;
	rc_encoder rc;
	rc_decoder rd;
	rc_model rcm;
	FILE *fp;
	
	op = (uc *)malloc(BSIZE+1);
	s = (uc *)malloc(BSIZE+1);
	if (op && s) {
		for (i = 0; i < BSIZE; i++)
			s[i] = rand() % 16 + 248;
		
		n = ModelInit (&rcm, SYMS, ftbl, NULL, 4, (Uint)1<<14, ADAPT);
		if (n != RC_OK) {
			fprintf(stderr, "Error: maxFreq or totFreq > 65536\n");
			free(op);
			free(s);
			exit(1);
		}
		fp = fopen("test.tst", "wb");
		if (fp == NULL) {
			fprintf(stderr, "Error: Couldn't create test.tst\n");
			free(op);
			free(s);
			exit(1);
		}
		fprintf(stderr, "Encoding...\n");
		fflush(stdout);
		StartEncode (&rc, fp, NULL, 0);
		for (i = 0; i < BSIZE; i++) {
			EncodeSymbol (&rc, &rcm, s[i]);
			switch (rc.error) {
				case RC_OK:
					break;
				case RC_ERROR:
					fprintf(stderr, "Encode error: Symbol outside range\n");
					free(op);
					free(s);
					exit(1);
				case RC_IO_ERROR:
					fprintf(stderr, "Encode error: Could not output compressed data\n");
					free(op);
					free(s);
					exit(1);
			}
		}
		FinishEncode (&rc);
		fclose(fp);
		
		n = ModelInit (&rcm, SYMS, ftbl, NULL, 4, (Uint)1<<14, ADAPT);
		if (n != RC_OK) {
			fprintf(stderr, "Error: maxFreq or totFreq > 65536\n");
			free(op);
			free(s);
			exit(1);
		}
		fp = fopen("test.tst", "rb");
		fprintf(stderr, "Decoding...\n");
		fflush(stdout);
		StartDecode (&rd, fp, NULL, 0);
		n = 0;
		for (i = 0; i < BSIZE; i++) {
			sym = DecodeSymbol (&rd, &rcm);
			switch (rd.error) {
				case RC_OK:
					break;
				case RC_ERROR:
					fprintf(stderr, "Decode error: Symbol outside range\n");
					free(op);
					free(s);
					exit(1);
				case RC_IO_ERROR:
					fprintf(stderr, "Decode error: Could not input compressed data\n");
					free(op);
					free(s);
					exit(1);
			}
			if (s[n++] != sym) break;
		}
		
		fclose(fp);
		fprintf(stderr, "Done!\n");
		fflush(stdout);
		if (i == BSIZE) {
			fprintf(stderr, "Data length: %ld, Compressed length: %lu\n", (long)BSIZE, (unsigned long)rc.passed);
			fprintf(stderr, "Strings are the same!\n");
		}
		else
			fprintf(stderr, "Strings differ at %d...\n", i);
	}
	free(op);
	free(s);
	
	return 0;
}
