#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "range.h"

#define SYMS 256
/* Change nr. of models here, e.g. 1<<11 means 2048 models */
#define MODELS (Uint)(1<<11)
#define MASK (Uint)(MODELS-1)
#define IOBUF_SIZE 32*1024
#define COUNT 256

Uint ftbl[MODELS][SYMS], ftblp[SYMS];
rc_model rcm[MODELS], rcp;
char ibuf[IOBUF_SIZE];

/* An order 1 or 2 encoder file to file demonstration */
int main (int argc, char **argv) {
	int sym;
	rc_encoder rc;
	FILE *fp, *ofp;
	Uint fsize, i, n, s1, s2, c, p, flag;
	uc lastsym, lastsym2, lastsym3;
	
	if (argc != 3) {
		fprintf(stderr, "Usage:\n%s infile outfile\n", argv[0]);
		exit(1);
	}
	fp = fopen(argv[1], "rb");
	ofp = fopen(argv[2], "wb");
	if (fp && ofp) {
		setvbuf(fp, ibuf, _IOFBF, IOBUF_SIZE);
		if (fseek(fp, 0L, SEEK_END) != 0) {
			fprintf(stderr, "This program only works on files\n");
			exit(1);
		}
		fsize = ftell(fp);
		rewind(fp);
		putc(fsize & 0xff, ofp);  /* Save filesize */
		putc((fsize >> 8) & 0xff, ofp);
		putc((fsize >> 16) & 0xff, ofp);
		putc((fsize >> 24) & 0xff, ofp);
		
		/* Initiate all the models */
		for (n = 0; n < MODELS; n++) {
			i = ModelInit (&rcm[n], SYMS, ftbl[n], NULL, 100, (Uint)1<<14, ADAPT);
			if (i != RC_OK) {
				fprintf(stderr, "Error: maxFreq or totFreq > 65536\n");
				exit(1);
			}
		}
		i = ModelInit (&rcp, SYMS, ftblp, NULL, 1, (Uint)1<<16, ADAPT);
		if (i != RC_OK) {
			fprintf(stderr, "Error: maxFreq or totFreq > 65536\n");
			exit(1);
		}
		StartEncode (&rc, ofp, NULL, 0);
		printf("Encoding...\n");
		fflush(stdout);
		c = p = s1 = s2 = 0;
		lastsym = lastsym2 = lastsym3 = 0;
		flag = 1;
		while ((sym = fgetc(fp)) != EOF) {
			if (flag) {
				/* If data is text-like then use more bits */
				if (s1 > s2) {
					n = (lastsym^lastsym2<<5^lastsym3<<10) & MASK;
					EncodeSymbol (&rc, &rcm[n], sym);
					rcm[n].maxFreq = (Uint)1<<16;
				}
				else {
					n = (lastsym^lastsym2<<8) & MASK;
					EncodeSymbol (&rc, &rcm[n], sym);
					rcm[n].maxFreq = (Uint)1<<14;
				}
			}
			else
				EncodeSymbol (&rc, &rcp, sym);
			if (rc.error != RC_OK) break;
			c++;
			if (c >= COUNT) {
				c = 0;
				if (rc.passed >= p + COUNT)
					flag = 0;
				else
					flag = 1;
				p = rc.passed;
			}
			if (sym < 128) s1++;
			else           s2+=8;
			if (s1 + s2 > 8192) {
				s1 /= 2;
				s2 /= 2;
			}
			lastsym3 = lastsym2;
			lastsym2 = lastsym;
			lastsym = sym;
		}
		FinishEncode (&rc);
		
		switch (rc.error) {
			case RC_OK:
				printf("Compressed %lu bytes to %lu bytes\n", (unsigned long)fsize, (unsigned long)rc.passed);
				fflush(stdout);
				break;
			case RC_ERROR:
				fprintf(stderr, "Encode error: Symbol outside range\n");
				break;
			case RC_IO_ERROR:
				fprintf(stderr, "Encode error: Could not output compressed data\n");
		}
	}
	fclose(fp);
	fclose(ofp);
	
	return 0;
}
